@@ -48,26 +48,26 @@
 /* .PP
 /*	Configuration parameters:
 /* .IP user
-/* 	Username for connecting to the database.
+/*	Username for connecting to the database.
 /* .IP password
 /*	Password for the above.
 /* .IP dbname
 /*	Name of the database.
 /* .IP domain
-/*      List of domains the queries should be restricted to.  If
-/*      specified, only FQDN addresses whose domain parts matching this
-/*      list will be queried against the SQL database.  Lookups for
-/*      partial addresses are also supressed.  This can significantly
-/*      reduce the query load on the server.
+/*	List of domains the queries should be restricted to.  If
+/*	specified, only FQDN addresses whose domain parts matching this
+/*	list will be queried against the SQL database.  Lookups for
+/*	partial addresses are also supressed.  This can significantly
+/*	reduce the query load on the server.
 /* .IP query
-/*      Query template, before the query is actually issued, variable
+/*	Query template, before the query is actually issued, variable
 /*	substitutions are performed. See mysql_table(5) for details. If
 /*	No query is specified, the legacy variables \fItable\fR,
 /*	\fIselect_field\fR, \fIwhere_field\fR and \fIadditional_conditions\fR
 /*	are used to construct the query template.
 /* .IP result_format
-/*      The format used to expand results from queries.  Substitutions
-/*      are performed as described in mysql_table(5). Defaults to returning
+/*	The format used to expand results from queries.  Substitutions
+/*	are performed as described in mysql_table(5). Defaults to returning
 /*	the lookup result unchanged.
 /* .IP expansion_limit
 /*	Limit (if any) on the total number of lookup result values. Lookups which
@@ -91,6 +91,25 @@
 /*	releases.
 /* .IP hosts
 /*	List of hosts to connect to.
+/* .IP option_file
+/*	Read options from the given file instead of the default my.cnf
+/*	location.
+/* .IP option_group
+/*	Read options from the given group.
+/* .IP tls_cert_file
+/*	File containing client's X509 certificate.
+/* .IP tls_key_file
+/*	File containing the private key corresponding to \fItls_cert_file\fR.
+/* .IP tls_CAfile
+/*	File containing certificates for all of the X509 Certification
+/*	Authorities the client will recognize.  Takes precedence over
+/*	\fItls_CApath\fR.
+/* .IP tls_CApath
+/*	Directory containing X509 Certification Authority certificates
+/*	in separate individual files.
+/* .IP tls_verify_cert
+/*	Verify that the server's name matches the common name of the
+/*	certificate.
 /* .PP
 /*	For example, if you want the map to reference databases of
 /*	the name "your_db" and execute a query like this: select
@@ -114,7 +133,7 @@
 /* .br
 /*	hosts = host1.some.domain\fR \fBhost2.some.domain
 /* .IP additional_conditions
-/*      Backward compatibility when \fIquery\fR is not set, additional
+/*	Backward compatibility when \fIquery\fR is not set, additional
 /*	conditions to the WHERE clause.
 /* .IP hosts
 /*	List of hosts to connect to.
@@ -217,6 +236,8 @@ typedef struct {
     CFG_PARSER *parser;
     char   *query;
     char   *result_format;
+    char   *option_file;
+    char   *option_group;
     void   *ctx;
     int     expansion_limit;
     char   *username;
@@ -226,6 +247,14 @@ typedef struct {
     PLMYSQL *pldb;
 #if defined(MYSQL_VERSION_ID) && MYSQL_VERSION_ID >= 40000
     HOST   *active_host;
+    char   *tls_cert_file;
+    char   *tls_key_file;
+    char   *tls_CAfile;
+    char   *tls_CApath;
+    char   *tls_ciphers;
+#if MYSQL_VERSION_ID >= 50023
+    int     tls_verify_cert;
+#endif
 #endif
 } DICT_MYSQL;
 
@@ -242,12 +271,11 @@ typedef struct {
 
 /* internal function declarations */
 static PLMYSQL *plmysql_init(ARGV *);
-static MYSQL_RES *plmysql_query(DICT_MYSQL *, const char *, VSTRING *, char *,
-				        char *, char *);
+static MYSQL_RES *plmysql_query(DICT_MYSQL *, const char *, VSTRING *);
 static void plmysql_dealloc(PLMYSQL *);
 static void plmysql_close_host(HOST *);
 static void plmysql_down_host(HOST *);
-static void plmysql_connect_single(HOST *, char *, char *, char *);
+static void plmysql_connect_single(DICT_MYSQL *, HOST *);
 static const char *dict_mysql_lookup(DICT *, const char *);
 DICT   *dict_mysql_open(const char *, int, int);
 static void dict_mysql_close(DICT *);
@@ -321,9 +349,11 @@ static const char *dict_mysql_lookup(DICT *dict, const char *name)
 	    msg_info("%s: Skipping lookup of '%s'", myname, name);
 	return (0);
     }
-    if (domain_rc < 0)
+    if (domain_rc < 0) {
+	msg_warn("%s:%s 'domain' pattern match failed for '%s'",
+		 dict->type, dict->name, name);
 	DICT_ERR_VAL_RETURN(dict, domain_rc, (char *) 0);
-
+    }
 #define INIT_VSTR(buf, len) do { \
 	if (buf == 0) \
 	    buf = vstring_alloc(len); \
@@ -349,10 +379,7 @@ static const char *dict_mysql_lookup(DICT *dict, const char *name)
 	return (0);
 
     /* do the query - set dict->error & cleanup if there's an error */
-    if ((query_res = plmysql_query(dict_mysql, name, query,
-				   dict_mysql->dbname,
-				   dict_mysql->username,
-				   dict_mysql->password)) == 0) {
+    if ((query_res = plmysql_query(dict_mysql, name, query)) == 0) {
 	dict->error = DICT_ERR_RETRY;
 	return (0);
     }
@@ -428,10 +455,10 @@ static HOST *dict_mysql_find_host(PLMYSQL *PLDB, unsigned stat, unsigned type)
 
 /* dict_mysql_get_active - get an active connection */
 
-static HOST *dict_mysql_get_active(PLMYSQL *PLDB, char *dbname,
-				           char *username, char *password)
+static HOST *dict_mysql_get_active(DICT_MYSQL *dict_mysql)
 {
     const char *myname = "dict_mysql_get_active";
+    PLMYSQL *PLDB = dict_mysql->pldb;
     HOST   *host;
     int     count = RETRY_CONN_MAX;
 
@@ -457,7 +484,7 @@ static HOST *dict_mysql_get_active(PLMYSQL *PLDB, char *dbname,
 	if (msg_verbose)
 	    msg_info("%s: attempting to connect to host %s", myname,
 		     host->hostname);
-	plmysql_connect_single(host, dbname, username, password);
+	plmysql_connect_single(dict_mysql, host);
 	if (host->stat == STATACTIVE)
 	    return host;
     }
@@ -468,7 +495,7 @@ static HOST *dict_mysql_get_active(PLMYSQL *PLDB, char *dbname,
 
 /* dict_mysql_event - callback: close idle connections */
 
-static void dict_mysql_event(int unused_event, char *context)
+static void dict_mysql_event(int unused_event, void *context)
 {
     HOST   *host = (HOST *) context;
 
@@ -485,17 +512,12 @@ static void dict_mysql_event(int unused_event, char *context)
 
 static MYSQL_RES *plmysql_query(DICT_MYSQL *dict_mysql,
 				        const char *name,
-				        VSTRING *query,
-				        char *dbname,
-				        char *username,
-				        char *password)
+				        VSTRING *query)
 {
-    PLMYSQL *PLDB = dict_mysql->pldb;
     HOST   *host;
     MYSQL_RES *res = 0;
 
-    while ((host = dict_mysql_get_active(PLDB, dbname, username, password)) != NULL) {
-
+    while ((host = dict_mysql_get_active(dict_mysql)) != NULL) {
 #if defined(MYSQL_VERSION_ID) && MYSQL_VERSION_ID >= 40000
 
 	/*
@@ -517,7 +539,7 @@ static MYSQL_RES *plmysql_query(DICT_MYSQL *dict_mysql,
 	    } else {
 		if (msg_verbose)
 		    msg_info("dict_mysql: successful query from host %s", host->hostname);
-		event_request_timer(dict_mysql_event, (char *) host, IDLE_CONN_INTV);
+		event_request_timer(dict_mysql_event, (void *) host, IDLE_CONN_INTV);
 		break;
 	    }
 	} else {
@@ -534,15 +556,32 @@ static MYSQL_RES *plmysql_query(DICT_MYSQL *dict_mysql,
  * used to reconnect to a single database when one is down or none is
  * connected yet. Log all errors and set the stat field of host accordingly
  */
-static void plmysql_connect_single(HOST *host, char *dbname, char *username, char *password)
+static void plmysql_connect_single(DICT_MYSQL *dict_mysql, HOST *host)
 {
     if ((host->db = mysql_init(NULL)) == NULL)
 	msg_fatal("dict_mysql: insufficient memory");
+    if (dict_mysql->option_file)
+	mysql_options(host->db, MYSQL_READ_DEFAULT_FILE, dict_mysql->option_file);
+    if (dict_mysql->option_group)
+	mysql_options(host->db, MYSQL_READ_DEFAULT_GROUP, dict_mysql->option_group);
+#if defined(MYSQL_VERSION_ID) && MYSQL_VERSION_ID >= 40000
+    if (dict_mysql->tls_key_file || dict_mysql->tls_cert_file ||
+	dict_mysql->tls_CAfile || dict_mysql->tls_CApath || dict_mysql->tls_ciphers)
+	mysql_ssl_set(host->db,
+		      dict_mysql->tls_key_file, dict_mysql->tls_cert_file,
+		      dict_mysql->tls_CAfile, dict_mysql->tls_CApath,
+		      dict_mysql->tls_ciphers);
+#if MYSQL_VERSION_ID >= 50023
+    if (dict_mysql->tls_verify_cert != -1)
+	mysql_options(host->db, MYSQL_OPT_SSL_VERIFY_SERVER_CERT,
+		      &dict_mysql->tls_verify_cert);
+#endif
+#endif
     if (mysql_real_connect(host->db,
 			   (host->type == TYPEINET ? host->name : 0),
-			   username,
-			   password,
-			   dbname,
+			   dict_mysql->username,
+			   dict_mysql->password,
+			   dict_mysql->dbname,
 			   host->port,
 			   (host->type == TYPEUNIX ? host->name : 0),
 			   0)) {
@@ -575,14 +614,14 @@ static void plmysql_down_host(HOST *host)
     host->db = 0;
     host->ts = time((time_t *) 0) + RETRY_CONN_INTV;
     host->stat = STATFAIL;
-    event_cancel_timer(dict_mysql_event, (char *) host);
+    event_cancel_timer(dict_mysql_event, (void *) host);
 }
 
 /* mysql_parse_config - parse mysql configuration file */
 
 static void mysql_parse_config(DICT_MYSQL *dict_mysql, const char *mysqlcf)
 {
-    const char *myname = "mysqlname_parse";
+    const char *myname = "mysql_parse_config";
     CFG_PARSER *p = dict_mysql->parser;
     VSTRING *buf;
     char   *hosts;
@@ -591,6 +630,18 @@ static void mysql_parse_config(DICT_MYSQL *dict_mysql, const char *mysqlcf)
     dict_mysql->password = cfg_get_str(p, "password", "", 0, 0);
     dict_mysql->dbname = cfg_get_str(p, "dbname", "", 1, 0);
     dict_mysql->result_format = cfg_get_str(p, "result_format", "%s", 1, 0);
+    dict_mysql->option_file = cfg_get_str(p, "option_file", NULL, 0, 0);
+    dict_mysql->option_group = cfg_get_str(p, "option_group", NULL, 0, 0);
+#if defined(MYSQL_VERSION_ID) && MYSQL_VERSION_ID >= 40000
+    dict_mysql->tls_key_file = cfg_get_str(p, "tls_key_file", NULL, 0, 0);
+    dict_mysql->tls_cert_file = cfg_get_str(p, "tls_cert_file", NULL, 0, 0);
+    dict_mysql->tls_CAfile = cfg_get_str(p, "tls_CAfile", NULL, 0, 0);
+    dict_mysql->tls_CApath = cfg_get_str(p, "tls_CApath", NULL, 0, 0);
+    dict_mysql->tls_ciphers = cfg_get_str(p, "tls_ciphers", NULL, 0, 0);
+#if MYSQL_VERSION_ID >= 50023
+    dict_mysql->tls_verify_cert = cfg_get_bool(p, "tls_verify_cert", -1);
+#endif
+#endif
 
     /*
      * XXX: The default should be non-zero for safety, but that is not
@@ -632,7 +683,7 @@ static void mysql_parse_config(DICT_MYSQL *dict_mysql, const char *mysqlcf)
 
     hosts = cfg_get_str(p, "hosts", "", 0, 0);
 
-    dict_mysql->hosts = argv_split(hosts, " ,\t\r\n");
+    dict_mysql->hosts = argv_split(hosts, CHARS_COMMA_SP);
     if (dict_mysql->hosts->argc == 0) {
 	argv_add(dict_mysql->hosts, "localhost", ARGV_END);
 	argv_terminate(dict_mysql->hosts);
@@ -759,6 +810,22 @@ static void dict_mysql_close(DICT *dict)
     myfree(dict_mysql->dbname);
     myfree(dict_mysql->query);
     myfree(dict_mysql->result_format);
+    if (dict_mysql->option_file)
+	myfree(dict_mysql->option_file);
+    if (dict_mysql->option_group)
+	myfree(dict_mysql->option_group);
+#if defined(MYSQL_VERSION_ID) && MYSQL_VERSION_ID >= 40000
+    if (dict_mysql->tls_key_file)
+	myfree(dict_mysql->tls_key_file);
+    if (dict_mysql->tls_cert_file)
+	myfree(dict_mysql->tls_cert_file);
+    if (dict_mysql->tls_CAfile)
+	myfree(dict_mysql->tls_CAfile);
+    if (dict_mysql->tls_CApath)
+	myfree(dict_mysql->tls_CApath);
+    if (dict_mysql->tls_ciphers)
+	myfree(dict_mysql->tls_ciphers);
+#endif
     if (dict_mysql->hosts)
 	argv_free(dict_mysql->hosts);
     if (dict_mysql->ctx)
@@ -774,16 +841,16 @@ static void plmysql_dealloc(PLMYSQL *PLDB)
     int     i;
 
     for (i = 0; i < PLDB->len_hosts; i++) {
-	event_cancel_timer(dict_mysql_event, (char *) (PLDB->db_hosts[i]));
+	event_cancel_timer(dict_mysql_event, (void *) (PLDB->db_hosts[i]));
 	if (PLDB->db_hosts[i]->db)
 	    mysql_close(PLDB->db_hosts[i]->db);
 	myfree(PLDB->db_hosts[i]->hostname);
 	if (PLDB->db_hosts[i]->name)
 	    myfree(PLDB->db_hosts[i]->name);
-	myfree((char *) PLDB->db_hosts[i]);
+	myfree((void *) PLDB->db_hosts[i]);
     }
-    myfree((char *) PLDB->db_hosts);
-    myfree((char *) (PLDB));
+    myfree((void *) PLDB->db_hosts);
+    myfree((void *) (PLDB));
 }
 
 #endif
