@@ -130,16 +130,20 @@ static MAPS *tls_per_site;		/* lookup table(s) */
 void    smtp_tls_list_init(void)
 {
     if (*var_smtp_tls_policy) {
-	tls_policy = maps_create(SMTP_X(TLS_POLICY), var_smtp_tls_policy,
-				 DICT_FLAG_LOCK | DICT_FLAG_FOLD_FIX);
+	tls_policy = maps_create(VAR_LMTP_SMTP(TLS_POLICY),
+				 var_smtp_tls_policy,
+				 DICT_FLAG_LOCK | DICT_FLAG_FOLD_FIX
+				 | DICT_FLAG_UTF8_REQUEST);
 	if (*var_smtp_tls_per_site)
 	    msg_warn("%s ignored when %s is not empty.",
-		     SMTP_X(TLS_PER_SITE), SMTP_X(TLS_POLICY));
+		     VAR_LMTP_SMTP(TLS_PER_SITE), VAR_LMTP_SMTP(TLS_POLICY));
 	return;
     }
     if (*var_smtp_tls_per_site) {
-	tls_per_site = maps_create(SMTP_X(TLS_PER_SITE), var_smtp_tls_per_site,
-				   DICT_FLAG_LOCK | DICT_FLAG_FOLD_FIX);
+	tls_per_site = maps_create(VAR_LMTP_SMTP(TLS_PER_SITE),
+				   var_smtp_tls_per_site,
+				   DICT_FLAG_LOCK | DICT_FLAG_FOLD_FIX
+				   | DICT_FLAG_UTF8_REQUEST);
     }
 }
 
@@ -240,7 +244,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
     }
     saved_policy = policy = mystrdup(lookup);
 
-    if ((tok = mystrtok(&policy, "\t\n\r ,")) == 0) {
+    if ((tok = mystrtok(&policy, CHARS_COMMA_SP)) == 0) {
 	msg_warn("%s: invalid empty policy", WHERE);
 	INVALID_RETURN(tls->why, site_level);
     }
@@ -255,7 +259,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
      * Warn about ignored attributes when TLS is disabled.
      */
     if (*site_level < TLS_LEV_MAY) {
-	while ((tok = mystrtok(&policy, "\t\n\r ,")) != 0)
+	while ((tok = mystrtok(&policy, CHARS_COMMA_SP)) != 0)
 	    msg_warn("%s: ignoring attribute \"%s\" with TLS disabled",
 		     WHERE, tok);
 	FREE_RETURN;
@@ -265,7 +269,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
      * Errors in attributes may have security consequences, don't ignore
      * errors that can degrade security.
      */
-    while ((tok = mystrtok(&policy, "\t\n\r ,")) != 0) {
+    while ((tok = mystrtok(&policy, CHARS_COMMA_SP)) != 0) {
 	if ((err = split_nameval(tok, &name, &val)) != 0) {
 	    msg_warn("%s: malformed attribute/value pair \"%s\": %s",
 		     WHERE, tok, err);
@@ -395,7 +399,7 @@ static int load_tas(TLS_DANE *dane, const char *files)
     char   *file;
 
     do {
-	if ((file = mystrtok(&buf, "\t\n\r ,")) != 0)
+	if ((file = mystrtok(&buf, CHARS_COMMA_SP)) != 0)
 	    ret = tls_dane_load_trustfile(dane, file);
     } while (file && ret);
 
@@ -433,6 +437,7 @@ static void set_cipher_grade(SMTP_TLS_POLICY *tls)
 	break;
 
     case TLS_LEV_DANE:
+    case TLS_LEV_DANE_ONLY:
     case TLS_LEV_FPRINT:
     case TLS_LEV_VERIFY:
     case TLS_LEV_SECURE:
@@ -491,7 +496,7 @@ static void *policy_create(const char *unused_key, void *context)
     } else if (tls_per_site) {
 	tls_site_lookup(tls, &site_level, dest, "next-hop destination");
 	if (site_level != TLS_LEV_INVALID
-	    && strcasecmp(dest, host) != 0)
+	    && strcasecmp_utf8(dest, host) != 0)
 	    tls_site_lookup(tls, &site_level, host, "server hostname");
 
 	/*
@@ -516,9 +521,11 @@ static void *policy_create(const char *unused_key, void *context)
     switch (site_level) {
     default:
 	tls->level = site_level;
+	/* FALLTHROUGH */
     case TLS_LEV_NOTFOUND:
 	break;
     case TLS_LEV_INVALID:
+	tls->level = site_level;
 	return ((void *) tls);
     }
 
@@ -528,7 +535,7 @@ static void *policy_create(const char *unused_key, void *context)
      * "dane-only" changes to "dane" once we obtain the requisite TLSA
      * records.
      */
-    if (tls->level == TLS_LEV_DANE || tls->level == TLS_LEV_DANE_ONLY)
+    if (TLS_DANE_BASED(tls->level))
 	dane_init(tls, iter);
     if (tls->level == TLS_LEV_INVALID)
 	return ((void *) tls);
@@ -557,13 +564,14 @@ static void *policy_create(const char *unused_key, void *context)
     case TLS_LEV_MAY:
     case TLS_LEV_ENCRYPT:
     case TLS_LEV_DANE:
+    case TLS_LEV_DANE_ONLY:
 	break;
     case TLS_LEV_FPRINT:
 	if (tls->dane == 0)
 	    tls->dane = tls_dane_alloc();
 	if (!TLS_DANE_HASEE(tls->dane)) {
 	    tls_dane_add_ee_digests(tls->dane, var_smtp_tls_fpt_dgst,
-				    var_smtp_tls_fpt_cmatch, "\t\n\r, ");
+				    var_smtp_tls_fpt_cmatch, CHARS_COMMA_SP);
 	    if (!TLS_DANE_HASEE(tls->dane)) {
 		msg_warn("nexthop domain %s: configured at fingerprint "
 		       "security level, but with no fingerprints to match.",
@@ -579,7 +587,7 @@ static void *policy_create(const char *unused_key, void *context)
 	    tls->matchargv =
 		argv_split(tls->level == TLS_LEV_VERIFY ?
 			   var_smtp_tls_vfy_cmatch : var_smtp_tls_sec_cmatch,
-			   "\t\n\r, :");
+			   CHARS_COMMA_SP ":");
 	if (*var_smtp_tls_tafile) {
 	    if (tls->dane == 0)
 		tls->dane = tls_dane_alloc();
@@ -618,7 +626,7 @@ static void policy_delete(void *item, void *unused_context)
 	tls_dane_free(tls->dane);
     dsb_free(tls->why);
 
-    myfree((char *) tls);
+    myfree((void *) tls);
 }
 
 /* smtp_tls_policy_cache_query - cached lookup of TLS policy */
@@ -838,7 +846,6 @@ static void dane_init(SMTP_TLS_POLICY *tls, SMTP_ITERATOR *iter)
     } else if (!TLS_DANE_HASEE(dane))
 	msg_panic("empty DANE match list");
     tls->dane = dane;
-    tls->level = TLS_LEV_DANE;
     return;
 }
 
